from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin, UserAdmin
from django.contrib.auth.models import Group as DjangoGroup

from .models import Group, User


admin.site.unregister(DjangoGroup)
admin.register(Group)(GroupAdmin)

admin.register(User)(UserAdmin)



"""
Table plugin
############

Plugin for Django CMS QE providing table listing. It allows to specify
table, columns, paging and all other attributes including CSS classes
to easily add any view to database without needs to create new plugin.

Usage
*****

Add plugin from section **Generic** called **Table**.

At configuration screen pick which table you want to use. Then specify which
columns to show, in which order and how many items per page to show.

TODO: Later there will be provided options to pre-filter and possibility to
filter on client side by user.

API
***

Django CMS
==========

.. autoclass:: cms_qe_table.cms_plugins.TablePlugin
    :members:

.. autoclass:: cms_qe_table.models.TablePluginModel
    :members:

.. autoclass:: cms_qe_table.forms.TablePluginForm
    :members:

Views
=====

.. automodule:: cms_qe_table.views
    :members:

.. automodule:: cms_qe_table.templatetags.cms_qe_table_filters
    :members:

Utils
=====

.. automodule:: cms_qe_table.utils
    :members:

Exceptions
==========

.. automodule:: cms_qe_table.exceptions
    :members:

"""



# pylint: disable=invalid-name
from unittest.mock import Mock

import pytest
from filer.models import File
from pytest_data import get_data

from .models import HostingVideoPlayer, SourceFileVideoPlayer


@pytest.fixture
def cms_qe_video_source_file_video_player_model(request):
    file_mock = Mock(spec=File, name='FileMock')
    file_mock._state = Mock()

    return SourceFileVideoPlayer(
        **get_data(request, 'cms_qe_video_source_file_video_player_model_data', {'source_file': file_mock})
    )


@pytest.fixture
def cms_qe_video_hosting_video_player_model(request):
    return HostingVideoPlayer(**get_data(
        request,
        'cms_qe_video_hosting_video_player_model_data',
    ))



"""
Settings is stored in ``cms_qe.settings``. You can find few prepared ready-to-use environments:

 * prod,
 * dev
 * and test.

First one is for production use. It means at your app you should just import everything from
there and add what you have to change (your paths and database for example). Actually the
minimum config is for example this one:

.. code-block:: python

    import os

    from cms_qe.settings.prod import *


    INSTALLED_APPS += [
        'example',
    ]

    ROOT_URLCONF = 'example.urls'
    WSGI_APPLICATION = 'example.wsgi.application'

    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    STATIC_ROOT = os.path.join(BASE_DIR, 'static')
    MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
        }
    }

Security
--------

.. automodule:: cms_qe.settings.base.security

"""



"""
Auth plugin
###########

Plugins for Django CMS QE providing authentication. It allows
user to register and login into site.

Usage
*****

Authentication forms are activated by default and is possible to use at
**/auth/login** or **/auth/register** but you can also use login or
registration form anywhere you want. Just from section **Generic** use
**Login form** or **Register form**.

API
***

Django CMS
==========

.. autoclass:: cms_qe_auth.cms_plugins.LoginFormPlugin
    :members:

.. autoclass:: cms_qe_auth.cms_plugins.RegisterFormPlugin
    :members:

.. autoclass:: cms_qe_auth.cms_menus.AuthModifier
    :members:

Models
======

.. autoclass:: cms_qe_auth.models.User
    :members:

Views
=====

.. automodule:: cms_qe_auth.views
    :members:

.. automodule:: cms_qe_auth.forms
    :members:
"""

default_app_config = 'cms_qe_auth.apps.AuthConfig'



"""
Dummy testing template tags and filters.
"""

from __future__ import absolute_import

from django.template import Library, Node, TemplateSyntaxError

from analytical.templatetags.analytical import TAG_LOCATIONS


register = Library()


def _location_node(location):
    class DummyNode(Node):
        def render(self, context):
            return "<!-- dummy_%s -->" % location
    return DummyNode

_location_nodes = dict((l, _location_node(l)) for l in TAG_LOCATIONS)


def _location_tag(location):
    def dummy_tag(parser, token):
        bits = token.split_contents()
        if len(bits) > 1:
            raise TemplateSyntaxError("'%s' tag takes no arguments" % bits[0])
        return _location_nodes[location]
    return dummy_tag

for loc in TAG_LOCATIONS:
    register.tag('dummy_%s' % loc, _location_tag(loc))


def contribute_to_analytical(add_node_cls):
    for location in TAG_LOCATIONS:
        add_node_cls(location, _location_nodes[location])



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _

from .models import MenuPluginModel


@plugin_pool.register_plugin
class MenuPlugin(CMSPluginBase):
    """
    CMS plugin allowing to add menu at any place.
    """

    model = MenuPluginModel
    module = _('Menu')
    name = _('Menu')
    render_template = 'cms_qe/menu/menu_tag.html'
    cache = False




from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool


@plugin_pool.register_plugin
class BreadcrumbPlugin(CMSPluginBase):
    """
    CMS plugin allowing to add breadcrumb at any place.
    """

    name = _('Breadcrumb')
    render_template = 'cms_qe/breadcrumb/breadcrumb_tag.html'
    cache = False



"""
Mailing settings, by default app looks for smtp server.
"""

EMAIL_HOST = 'localhost'
EMAIL_PORT = 587  # TLS uses usually 587, not 22

EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''

EMAIL_USE_TLS = True  # Prefer to use secure mailing by default
EMAIL_SUBJECT_PREFIX = ''  # Remove Django default prefix

DEFAULT_FROM_EMAIL = 'django_cms_qe@localhost'



from ..models import MailingList


def test_get_lists_from_mailchimp(mock_mailchimp, admin_client):
    assert MailingList.objects.count() == 0
    admin_client.get('/cms-qe/newsletter/sync-lists')
    assert MailingList.objects.count() == 2


def test_get_lists_from_mailchimp_not_staff(mock_mailchimp, client):
    assert MailingList.objects.count() == 0
    client.get('/cms-qe/newsletter/sync-lists')
    assert MailingList.objects.count() == 0



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend


class TestAuthBackend(ModelBackend):
    def authenticate(self, username=None, password=None):
        User = get_user_model()
        try:
            user = User.objects.get(username=username)
            return user
        except User.DoesNotExist:
            return



from typing import Type, TypeVar

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login
from django.http import HttpRequest
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.utils.translation import ugettext as _

from cms_qe.utils import get_base_url
from cms_qe_auth.utils import get_user_by_uidb64

from .forms import RegisterForm

RF = TypeVar('RF', bound=RegisterForm)


def register(
        request: HttpRequest,
        template_name: str = 'cms_qe/auth/register.html',
        register_form: Type[RF] = RegisterForm,
):
    """
    Displays the register form and handles the register action.
    """
    if request.method == 'POST':
        form = register_form(data=request.POST)
        if form.is_valid():
            base_url = get_base_url(request)
            user = form.save(commit=False)
            user.is_active = False
            user.save(base_url=base_url)
            messages.success(request, _('You were successfully registered. Please confirm your email address.'))
            return redirect('register')
        else:
            messages.error(request, _('Please correct errors below.'))
    else:
        form = register_form()

    context = {'form': form}
    return TemplateResponse(request, template_name, context)


def activate(
        request: HttpRequest,
        uidb64: str,
        token: str,
        template_name_complete: str = 'cms_qe/auth/email_confirmation_complete.html',
        template_name_fail: str = 'cms_qe/auth/email_confirmation_fail.html',
):
    user = get_user_by_uidb64(uidb64)
    if user and user.activate(token):
        # Use default django authentication backend if multiple authentication backends are configured
        # It doesn't work with custom authentication backends. If you use custom auth backend you should completely
        # rewrite authentication and activation.
        if len(settings.AUTHENTICATION_BACKENDS) > 1:
            user.backend = 'django.contrib.auth.backends.ModelBackend'
        login(request, user)
        return TemplateResponse(request, template_name_complete)
    return TemplateResponse(request, template_name_fail)



from unittest.mock import patch

import pytest
from django.test import override_settings
from mailqueue.models import MailerMessage
from pytest_data import use_data

from cms_qe.utils import get_base_url, get_email, get_functions, get_module, get_modules


@pytest.mark.parametrize('expect,module_name', [
    (True, 'views'),
    (True, 'views.monitoring'),
    (False, 'no_module'),
    (False, 'monitoring.get_status'),
])
def test_get_module(expect, module_name):
    module = get_module('cms_qe', module_name)
    assert module if expect else module is None


def test_get_modules():
    modules = get_modules('monitoring')
    assert [module for app_name, module in modules if app_name == 'cms_qe']


def test_get_functions():
    functions = get_functions('monitoring', 'get_status')
    assert [function for app_name, function in functions if app_name == 'cms_qe']


def test_get_functions_existing_module_without_function():
    functions = get_functions('monitoring', 'no_function')
    assert not list(functions)


@override_settings(META_SITE_PROTOCOL='https', )
def test_get_absolute_url(post_request_empty):
    request = post_request_empty
    mock_url = 'test.com'
    with patch('cms_qe.utils.get_current_site', return_value=mock_url):
        base_url = get_base_url(request)
    assert base_url == 'https://{}'.format(mock_url)


def test_sending_email_synchronously(email, mailoutbox):
    assert len(mailoutbox) == 0
    email.save()
    assert len(mailoutbox) == 1
    sent_mail = mailoutbox[0]
    assert 'Test content' in sent_mail.body
    assert 'Test' in sent_mail.subject


@override_settings(MAILQUEUE_QUEUE_UP=True)
def test_sending_email_asynchronously(email, mailoutbox):
    assert MailerMessage.objects.all().count() == 0
    email.save()
    assert len(mailoutbox) == 0
    assert MailerMessage.objects.all().count() == 1
    assert 'Test content' in MailerMessage.objects.all().first().content


@use_data(email_data={'to': ['test1@examile.com', 'test2@examile.com']})
def test_sending_email_synchronously_more_addressee(email, mailoutbox):
    email.save()
    assert len(mailoutbox) == 1
    sent_mail = mailoutbox[0]
    assert len(sent_mail.to) == 2


@use_data(email_data={'to': ['test1@examile.com', 'test2@examile.com']})
@override_settings(MAILQUEUE_QUEUE_UP=True)
def test_sending_email_asynchronously_more_to_addressee(email, mailoutbox):
    email.save()
    assert MailerMessage.objects.all().count() == 1
    assert MailerMessage.objects.all().first().to_address == 'test1@examile.com, test2@examile.com'




import pytest
from pytest_data import get_data

from .models import TablePluginModel


@pytest.fixture
def cms_qe_table_model(request):
    return TablePluginModel(**get_data(request, 'cms_qe_table_model_data', {
        'table': 'auth_user',
        'columns': ['username', 'password'],
        'paging_show': False,
        'paging_per_page': 20,
    }))




class WizardPage:
    def __init__(self, driver):
        self._driver = driver

    def open(self):
        self._driver.go_to('/cms_wizard/create/?language=en')
        return self

    def create_home_page(self):
        """
        Creates empty homepage for Django CMS. Usuful to create some
        at the beggining of tests so any test can start somewhere.
        """
        self._driver.get_elm(tag_name='form').submit()
        self._driver.get_elm(tag_name='form').fill_out_and_submit({
            '1-title': 'homepage',
        })



from django.contrib import admin

from .models import MailingList, Subscriber, SubscribeTask


@admin.register(MailingList)
class MailingListAdmin(admin.ModelAdmin):
    list_display = ('name', 'external_service', 'external_id')
    list_filter = ('external_service',)
    search_fields = ('name', 'external_id')


@admin.register(Subscriber)
class SubscriberAdmin(admin.ModelAdmin):
    list_display = ('mailing_list', 'email', 'first_name', 'last_name', 'external_id')
    list_filter = ('mailing_list__external_service', 'mailing_list')
    search_fields = ('email', 'first_name', 'last_name', 'external_id')
    fields = (
        'mailing_list',
        'email',
        ('first_name', 'last_name'),
        'external_id',
    )

    actions = ['delete_selected_and_unsubscribe']

    def get_actions(self, request):
        actions = super(SubscriberAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    # Django admin does not use model.delete() when multiple items selected in admin interface
    # pylint:disable=unused-argument
    def delete_selected_and_unsubscribe(self, request, queryset):
        for obj in queryset:
            obj.unsubscribe()
        queryset.delete()


@admin.register(SubscribeTask)
class SubscribeTaskAdmin(admin.ModelAdmin):
    list_display = ('type', 'created', 'attempts', 'mailing_list', 'email', 'external_id')
    list_filter = ('type', 'created', 'attempts')
    search_fields = ('email', 'external_id', 'last_error')
    readonly_fields = list_display + ('last_error', 'first_name', 'last_name')
    fieldsets = (
        (None, {
            'fields': (
                'type',
                'created',
                'attempts',
                'last_error',
            ),
        }),
        ('Data', {
            'fields': (
                'mailing_list',
                'email',
                ('first_name', 'last_name'),
                'external_id',
            ),
        }),
    )

    def has_add_permission(self, request):
        return False



from django import forms

from .models import HostingVideoPlayer


class HostingVideoPlayerForm(forms.ModelForm):
    """
    Adding ids (HTML attributes) to control(checkbox) and video_hosting_service(selection) to process with JS.
    Script with processing: templates/cms_qe/defaul/video_widget.html
    """

    class Meta:
        model = HostingVideoPlayer
        fields = '__all__'
        widgets = {
            'controls': forms.CheckboxInput(attrs={'id': 'django-cms-qe-vimeo-disabled'}),
            'video_hosting_service': forms.Select(attrs={'id': 'django-cms-qe-hosting-choices'})
        }



from django.contrib.auth import get_user_model
from django.db import models
from django.db.models import Q
import pytest

from ..exceptions import TableDoesNotExists
from ..utils import get_model_by_table, get_models_choices, get_table_choices, get_field_type, get_filter_params


def test_get_model_by_table():
    User = get_user_model()
    model = get_model_by_table('auth_user')
    assert model is User


def test_get_model_by_table_not_found():
    with pytest.raises(TableDoesNotExists):
        get_model_by_table('table_does_not_exist')


def test_get_all_models():
    choices = get_models_choices()
    choices_admin_group = [item[1] for item in choices if item[0] == 'admin'][0]
    assert choices_admin_group == (
        ('django_admin_log', 'LogEntry'),
    )


def test_get_table_choices():
    choices = get_table_choices('auth_user')
    assert 'columns' in choices
    assert ('username', 'username', 'string') in choices['columns']


@pytest.mark.parametrize('field, expected_type', [
    (models.AutoField(), 'integer'),
    (models.BigAutoField(), 'integer'),
    (models.BigIntegerField(), 'integer'),
    (models.BooleanField(), 'boolean'),
    (models.CharField(), 'string'),
    (models.DateField(), 'string'),
    (models.DateTimeField(), 'string'),
    (models.DecimalField(), 'float'),
    (models.EmailField(), 'string'),
    (models.FloatField(), 'float'),
    (models.ForeignKey('LogEntry'), 'string'),
    (models.IntegerField(), 'integer'),
    (models.GenericIPAddressField(), 'string'),
    (models.NullBooleanField(), 'boolean'),
    (models.PositiveIntegerField(), 'integer'),
    (models.PositiveSmallIntegerField(), 'integer'),
    (models.SlugField(), 'string'),
    (models.SmallIntegerField(), 'integer'),
    (models.TextField(), 'string'),
    (models.TimeField(), 'string'),
    (models.URLField(), 'string'),
    (models.UUIDField(), 'string'),
])
def test_get_field_type(field, expected_type):
    assert get_field_type(field) == expected_type


def test_get_filter_params():
    class ForeignModel(models.Model):
        text1 = models.CharField()
        text2 = models.CharField()
        flag = models.BooleanField()
        number = models.IntegerField()

    class Model(models.Model):
        other = models.ForeignKey(ForeignModel)
        name = models.CharField()
        active = models.BooleanField()
        age = models.IntegerField()

    args, kwds = get_filter_params(Model, {
        'other': 'abc',
        'name': 'name',
        'active': True,
        'age': 123,
        'non-existent-field': 'blah',
    })
    assert str(args) == str([
        Q() | Q(other__text1__icontains='abc') | Q(other__text2__icontains='abc')
    ])
    assert kwds == {
        'name__icontains': 'name',
        'active': True,
        'age': 123,
    }



import pytest
from cms_qe_auth.models import User
from django.contrib.auth import get_user_model
from pytest_data import get_data


@pytest.fixture
def user(request):
    data = get_data(
        request, 'user_data', {
            'username': 'user',
            'email': 'user@example.com',
            'is_active': True,
            'password': 'pass'
        }
    )
    password = data['password']
    user = User(**data)
    user.set_password(password)
    user.save(base_url='/')
    return user


@pytest.fixture
def admin_user(request):
    return get_user_model().objects.create_superuser(**get_data(request, 'admin_user_data', {
        'username': 'admin',
        'email': 'admin@example.com',
        'password': 'password',  # Keep it same with pytest-django.
    }))



"""
Tests for the Piwik template tags and filters.
"""

from django.contrib.auth import get_user_model
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings
from constance.test import override_config

from ..templatetags.piwik import PiwikNode
from ..tests.utils import TagTestCase
from ..utils import AnalyticalException


@override_config(PIWIK_DOMAIN_PATH='example.com', PIWIK_SITE_ID='345')
class PiwikTagTestCase(TagTestCase):
    """
    Tests for the ``piwik`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('piwik', 'piwik')
        self.assertTrue(' ? "https" : "http") + "://example.com/";' in r, r)
        self.assertTrue("_paq.push(['setSiteId', 345]);" in r, r)
        self.assertTrue('img src="http://example.com/piwik.php?idsite=345"'
                        in r, r)

    def test_node(self):
        r = PiwikNode().render(Context({}))
        self.assertTrue(' ? "https" : "http") + "://example.com/";' in r, r)
        self.assertTrue("_paq.push(['setSiteId', 345]);" in r, r)
        self.assertTrue('img src="http://example.com/piwik.php?idsite=345"'
                        in r, r)

    @override_config(PIWIK_DOMAIN_PATH='example.com/piwik',
                     PIWIK_SITE_ID='345')
    def test_domain_path_valid(self):
        r = self.render_tag('piwik', 'piwik')
        self.assertTrue(' ? "https" : "http") + "://example.com/piwik/";' in r,
                        r)

    @override_config(PIWIK_DOMAIN_PATH='example.com:1234',
                     PIWIK_SITE_ID='345')
    def test_domain_port_valid(self):
        r = self.render_tag('piwik', 'piwik')
        self.assertTrue(' ? "https" : "http") + "://example.com:1234/";' in r,
                        r)

    @override_config(PIWIK_DOMAIN_PATH='example.com:1234/piwik',
                     PIWIK_SITE_ID='345')
    def test_domain_port_path_valid(self):
        r = self.render_tag('piwik', 'piwik')
        self.assertTrue(' ? "https" : "http") + "://example.com:1234/piwik/";' in r,
                        r)

    @override_config(PIWIK_DOMAIN_PATH='')
    def test_no_domain(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_SITE_ID='')
    def test_no_siteid(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_SITE_ID='x')
    def test_siteid_not_a_number(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='http://www.example.com')
    def test_domain_protocol_invalid(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='example.com/')
    def test_domain_slash_invalid(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='example.com:123:456')
    def test_domain_multi_port(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='example.com:')
    def test_domain_incomplete_port(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='example.com:/piwik')
    def test_domain_uri_incomplete_port(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_config(PIWIK_DOMAIN_PATH='example.com:12df')
    def test_domain_port_invalid(self):
        self.assertEqual(PiwikNode().render({}), '')

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = PiwikNode().render(context)
        self.assertTrue(r.startswith(
            '<!-- Piwik disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    def test_uservars(self):
        context = Context({'piwik_vars': [(1, 'foo', 'foo_val'),
                                          (2, 'bar', 'bar_val', 'page'),
                                          (3, 'spam', 'spam_val', 'visit')]})
        r = PiwikNode().render(context)
        msg = 'Incorrect Piwik custom variable rendering. Expected:\n%s\nIn:\n%s'
        for var_code in ['_paq.push(["setCustomVariable", 1, "foo", "foo_val", "page"]);',
                         '_paq.push(["setCustomVariable", 2, "bar", "bar_val", "page"]);',
                         '_paq.push(["setCustomVariable", 3, "spam", "spam_val", "visit"]);']:
            self.assertIn(var_code, r, msg % (var_code, r))

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_default_usertrack(self):
        User = get_user_model()
        context = Context({
            'user': User(username='BDFL', first_name='Guido', last_name='van Rossum')
        })
        r = PiwikNode().render(context)
        msg = 'Incorrect Piwik user tracking rendering.\nNot found:\n%s\nIn:\n%s'
        var_code = '_paq.push(["setUserId", "BDFL"]);'
        self.assertIn(var_code, r, msg % (var_code, r))

    def test_piwik_usertrack(self):
        context = Context({
            'piwik_identity': 'BDFL'
        })
        r = PiwikNode().render(context)
        msg = 'Incorrect Piwik user tracking rendering.\nNot found:\n%s\nIn:\n%s'
        var_code = '_paq.push(["setUserId", "BDFL"]);'
        self.assertIn(var_code, r, msg % (var_code, r))

    def test_analytical_usertrack(self):
        context = Context({
            'analytical_identity': 'BDFL'
        })
        r = PiwikNode().render(context)
        msg = 'Incorrect Piwik user tracking rendering.\nNot found:\n%s\nIn:\n%s'
        var_code = '_paq.push(["setUserId", "BDFL"]);'
        self.assertIn(var_code, r, msg % (var_code, r))

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_disable_usertrack(self):
        User = get_user_model()
        context = Context({
            'user': User(username='BDFL', first_name='Guido', last_name='van Rossum'),
            'piwik_identity': None
        })
        r = PiwikNode().render(context)
        msg = 'Incorrect Piwik user tracking rendering.\nFound:\n%s\nIn:\n%s'
        var_code = '_paq.push(["setUserId", "BDFL"]);'
        self.assertNotIn(var_code, r, msg % (var_code, r))



import pytest

from .fixtures import *
from .fixtures.base import _driver


def pytest_addoption(parser):
    parser.addoption('--no-display', action='store_true', help='do not use virtual display')


def pytest_configure(config):
    config.webdriverwrapper_screenshot_path = '/tmp/testresults'



"""
Analytics service integration
=============================

Integration with Google-Analytic_ and Piwik_ services.

This module is made using the django-analytical_ project.

.. _Google-Analytic : https://www.google.com/analytics/
.. _Piwik: https://piwik.org/
.. _django-analytical: https://pythonhosted.org/django-analytical_


Usage
-----
You can find all settings for analytics services in **Administration › Constance › Config** page.

Google Analytic
---------------

Setting the property ID
::::::::::::::::::
Every website you track with Google Analytics gets its own property ID, and the google_analytics tag will include it in
the rendered Javascript code. You can find the web property ID on the overview page of your account.
ID should looks like **UA-123456-1**. Set **GOOGLE_ANALYTICS_PROPERTY_ID** in the config page.

If you do not set the site ID the tracking code will not be rendered.


Display Advertising
::::::::::::::::::
Display Advertising allows you to view Demographics and Interests reports, add Remarketing Lists and support
DoubleClick Campain Manager integration.

You can enable Display Advertising features by setting the **GOOGLE_ANALYTICS_DISPLAY_ADVERTISING**.

Tracking site speed
:::::::::::::::::::
You can view page load times in the Site Speed report by setting the **GOOGLE_ANALYTICS_SITE_SPEED**
configuration setting.

By default, page load times are not tracked.

Anonymize IPs
:::::::::::::
You can enable the IP anonymization feature by setting the **GOOGLE_ANALYTICS_ANONYMIZE_IP** configuration setting.

This may be mandatory for deployments in countries that have a firm policies concerning data privacy (e.g. Germany).
By default, IPs are not anonymized.

Sample Rate
:::::::::::
You can configure the Sample Rate feature by setting the **GOOGLE_ANALYTICS_SAMPLE_RATE** configuration setting.

The value is a percentage and can be between 0 and 100 and can be a string or decimal value of with upto two decimal
places.

Site Speed Sample Rate
::::::::::::::::::::::
You can configure the Site Speed Sample Rate feature by setting the **GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE**
configuration setting.

The value is a percentage and can be between 0 and 100 and can be a string or decimal value of with up to two decimal
places.

Session Cookie Timeout
::::::::::::::::::::::
You can configure the Session Cookie Timeout feature by setting the **GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT**
configuration setting:

The value is the session cookie timeout in milliseconds or 0 to delete the cookie when the browser is closed.

Visitor Cookie Timeout
::::::::::::::::::::::
You can configure the Visitor Cookie Timeout feature by setting the **GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT**
configuration setting.

The value is the visitor cookie timeout in milliseconds or 0 to delete the cookie when the browser is closed.

Piwik
-----
Before you can use the Piwik integration, you must first define domain name and optional URI path to your Piwik server,
as well as the Piwik ID of the website you’re tracking with your Piwik server, in your project settings.

Setting the domain
::::::::::::::::::
Your Django project needs to know where your Piwik server is located. Typically, you’ll have Piwik installed
on a subdomain of its own (e.g. piwik.example.com), otherwise it runs in a subdirectory of a website of yours
(e.g. www.example.com/piwik). Set **PIWIK_DOMAIN_PATH** in the config page:

If you do not set a domain the tracking code will not be rendered.

Setting the site ID
:::::::::::::::::::
Your Piwik server can track several websites. Each website has its site ID (this is the idSite parameter in the query
string of your browser’s address bar when you visit the Piwik Dashboard). Set **PIWIK_SITE_ID** in the config page.

If you do not set the site ID the tracking code will not be rendered.
"""

# The code for this application was copied from https://github.com/jcassee/django-analytical v2.2.2
# and fixed to work with constance

__author__ = "Joost Cassee"
__email__ = "joost@cassee.net"
__version__ = "2.2.2"
__copyright__ = "Copyright (C) 2011-2016 Joost Cassee and others"
__license__ = "MIT License"



import os

from django.conf import settings
import pytest
from webdriverwrapper.pytest import *
from selenium import webdriver

from ..browser import ChromeBrowser
from ..pages.cms import LoginPage, WizardPage

__all__ = (
    'display',
    'session_driver',
    'driver',
    '_driver',
    'homepage_url',
    'admin_username',
)


@pytest.fixture(scope='session', autouse=True)
def display(request):
    no_display = request.config.getoption('--no-display')

    if no_display:
        yield
    else:
        from pyvirtualdisplay import Display

        display = Display(visible=0, size=(1200, 2000))
        display.start()
        yield
        display.stop()


@pytest.yield_fixture(scope='session')
def session_driver(request, homepage_url, admin_username):
    no_display = request.config.getoption('--no-display')

    driver = open_browser(homepage_url)
    try:
        LoginPage(driver).open().login(admin_username)
        WizardPage(driver).open().create_home_page()
        yield driver
    finally:
        if not no_display:
            driver.quit()


def open_browser(homepage_url):
    """
    Open browser a type URL `homepage_url`.
    """
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--incognito')
    chrome_options.add_argument('--no-sandbox')  # So it will work in GitLab CI.

    driver = ChromeBrowser(chrome_options=chrome_options)
    driver.get(homepage_url)
    return driver


@pytest.fixture
def _driver(session_driver, homepage_url):
    session_driver.get(homepage_url)
    return session_driver


@pytest.fixture(scope='session')
def homepage_url():
    port = os.environ.get('WEB_PORT', '8000')
    return 'http://localhost:{}'.format(port)


@pytest.fixture(scope='session')
def admin_username():
    return os.environ.get('ADMIN_USER', 'admin')



from django.contrib.admin.views.decorators import staff_member_required
from django.shortcuts import redirect

from .external_services.sync import sync_mailing_lists


# pylint: disable=unused-argument
@staff_member_required
def update_lists(request):
    """
    Download mailing lists from external services like Mailchimp.

    Used in administration in custom button "Synchronize mailing list from external sources".
    """
    sync_mailing_lists()
    return redirect('admin:cms_qe_newsletter_mailinglist_changelist')



"""
Configuration for unit testing.
"""

from .dev import *  # pylint: disable=wildcard-import,unused-wildcard-import


# It's needed by django-pytest.
ROOT_URLCONF = 'cms_qe.urls'

# Use base template.
ALDRYN_BOILERPLATE_NAME = 'legacy'

# Speed up creating users in unittests.
PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.MD5PasswordHasher',
]

TEST_MAILCHIMP_USERNAME = 'cms-qe-test'
TEST_MAILCHIMP_API_KEY = '4b1f852f66a317a500a5ae711a9181be-us16'
TEST_MAILCHIMP_LIST_ID = 'b6b91697ec'



from django.core.exceptions import FieldDoesNotExist
from django.contrib import admin
from django.http import HttpResponse
from django.utils.encoding import force_text
from import_export import fields, resources, widgets

EXPORT_TYPES_TO_CONTENT_TYPES = {
    'csv': 'text/csv',
    'tsv': 'text/tab-separated-values',
    'xls': 'application/vnd.ms-excel',
    'xlsx': 'application/vnd.ms-excel',
    'ods': 'application/vnd.oasis.opendocument.spreadsheet',
    'json': 'application/json',
    'yaml': 'text/plain',
    'html': 'text/html',
}
"""
Map from supported export types to their content type
used for file to download.
"""


def register_export_action(export_type, label):
    """
    Helper to register export action to specific type. It dynamically
    creates action function and register it as ``admin.site.add_action``
    with passed label.
    """

    # pylint: disable=unused-argument
    # Warning: Some admin pages can add more arguments to actions.
    def wrapper(modeladmin, request, queryset, *args, **kwargs):
        return export_file(export_type, queryset)

    wrapper.__name__ = 'export_selected_objects_as_{}'.format(export_type)
    wrapper.short_description = label

    admin.site.add_action(wrapper, wrapper.__name__)

    return wrapper


def export_file(export_type, queryset):
    """
    Same as :any:`cms_qe.export.export_data` but wraps it into
    :any:`django.http.HttpResponse` as file to download.
    """
    if export_type not in EXPORT_TYPES_TO_CONTENT_TYPES:
        raise Exception('Type {} is not supported'.format(export_type))

    content_type = EXPORT_TYPES_TO_CONTENT_TYPES[export_type]
    data = export_data(export_type, queryset)
    response = HttpResponse(data, content_type=content_type)
    response['Content-Disposition'] = 'attachment; filename="export.{}"'.format(export_type)
    return response


def export_data(export_type, queryset):
    """
    Export data as ``export_type``. Supported are only those in
    constant ``EXPORT_TYPES_TO_CONTENT_TYPES``.
    """
    model_fields = queryset.model._meta.get_fields()

    class ObjectResourceMeta(resources.ModelDeclarativeMetaclass):
        """
        When model has field with choices, we want to export verbose names
        instead of keys in database. For that on resource object has to be
        field with it's list of choices. ``ModelResource`` uses metaclass
        to collect those data so we need to override that metaclass and
        pass it manually. Assign it to class after creation is late.
        """
        def __new__(mcs, name, bases, attrs):
            for field in model_fields:
                choices = getattr(field, 'choices', None)
                if not choices:
                    continue
                attrs[field.name] = fields.Field(
                    attribute=field.name,  # Warning: attribute has to be here.
                    widget=ChoicesWidget(choices),
                )
            return super().__new__(mcs, name, bases, attrs)

    # pylint: disable=no-init,too-few-public-methods
    class ObjectResource(resources.ModelResource, metaclass=ObjectResourceMeta):
        class Meta:
            model = queryset.model

            # After adding some extra fields, it will break order.
            # Django's ``get_fields`` keeps order but we need to exclude
            # all auto created back references.
            # Warning: ID is also auto created.
            export_order = [
                field.name
                for field in model_fields
                if not field.auto_created or field.name == 'id'
            ]

        def get_export_headers(self) -> list:
            """
            As header use verbose name which is better than database name.
            """
            return [self.get_model_field_verbose_name(field) for field in self.get_export_fields()]

        @classmethod
        def get_model_field_verbose_name(cls, field) -> str:
            name = cls.get_field_name(field)
            try:
                field = queryset.model._meta.get_field(name)
            except FieldDoesNotExist:
                pass
            else:
                name = force_text(field.verbose_name)
            return name

        def export_field(self, field, obj):
            value = super().export_field(field, obj)
            if '__proxy__' in value.__class__.__name__:
                value = force_text(value)
            return value

    dataset = ObjectResource().export(queryset)
    return getattr(dataset, export_type)


# Taken from https://github.com/django-import-export/django-import-export/issues/525#issuecomment-303046691
class ChoicesWidget(widgets.Widget):
    """
    Widget that uses choice display values in place of database values
    """

    # pylint:disable=unused-argument
    def __init__(self, choices, *args, **kwargs):
        """
        Creates a self.choices dict with a key, display value, and value,
        db value, e.g. {'Chocolate': 'CHOC'}
        """
        self.choices = dict(choices)
        self.revert_choices = dict((v, k) for k, v in self.choices.items())

    # pylint: disable=keyword-arg-before-vararg
    def clean(self, value, row=None, *args, **kwargs):
        """Returns the db value given the display value"""
        return self.revert_choices.get(value, value) if value else None

    def render(self, value, obj=None):
        """Returns the display value given the db value"""
        return self.choices.get(value, '')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-08 09:15
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cms_qe_table', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='tablepluginmodel',
            name='filter',
            field=jsonfield.fields.JSONField(default=dict, verbose_name='Filter'),
        ),
    ]



import json

from cms_qe.views import get_monitoring_data


def test_get_monitoring_data():
    data = get_monitoring_data()
    assert data['status']
    assert 'cms_qe' in data['app_details']


def test_monitoring_view(client):
    result = client.get('/api/monitoring').content
    data = json.loads(str(result, 'utf8'))
    assert data['status']



from django.core.exceptions import ValidationError
from pytest_data import use_data
import pytest

from ..models import Subscriber, SubscribeTask, SERVICE_MAILCHIMP


def test_clean_newsletter_plugin(cms_qe_newsletter_plugin):
    cms_qe_newsletter_plugin.fullname_show = False
    cms_qe_newsletter_plugin.fullname_require = True
    with pytest.raises(ValidationError):
        cms_qe_newsletter_plugin.clean()


def test_validate_mailing_list(cms_qe_mailing_list):
    cms_qe_mailing_list.external_service = SERVICE_MAILCHIMP
    cms_qe_mailing_list.external_id = None
    with pytest.raises(ValidationError):
        cms_qe_mailing_list.clean()


def test_save_will_create_task(cms_qe_mailing_list):
    sub = Subscriber(
        mailing_list=cms_qe_mailing_list,
        email='test@example.com',
    )
    assert SubscribeTask.objects.all().count() == 0
    sub.save()
    assert SubscribeTask.objects.all().count() == 1


@use_data(cms_qe_subscriber_data={'external_id': 'test_id'})
def test_delete_will_create_task(cms_qe_subscriber):
    assert SubscribeTask.objects.all().count() == 0
    cms_qe_subscriber.delete()
    assert SubscribeTask.objects.all().count() == 1


@use_data(cms_qe_subscriber_data={'external_id': None})
def test_delete_will_remove_pending_task(cms_qe_subscriber):
    assert SubscribeTask.objects.all().count() == 1
    cms_qe_subscriber.delete()
    assert SubscribeTask.objects.all().count() == 0



from cms import api
from cms.models import Placeholder
from cms.plugin_rendering import ContentRenderer
from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory
from sekizai.context import SekizaiContext


def render_plugin(plugin, path='/', **data):
    placeholder = Placeholder.objects.create(slot='test')
    model_instance = api.add_plugin(placeholder, plugin, 'en', **data)
    request = generate_get_request(path)
    renderer = ContentRenderer(request=request)
    context = SekizaiContext()
    context.update({'request': request, })
    html = renderer.render_plugin(model_instance, context)
    return html


def generate_get_request(path):
    request = RequestFactory().get(path)
    request.user = AnonymousUser()
    request.session = {}
    return request


def generate_post_request(path='', body=None):
    request = RequestFactory().post(path, body)
    request.user = AnonymousUser()
    request.session = {}
    messages = FallbackStorage(request)
    setattr(request, '_messages', messages)
    return request


# pylint: disable=dangerous-default-value
def create_page(title, language='en', page_params={}):
    page_params.setdefault('published', True)
    page_params.setdefault('overwrite_url', page_params.get('slug'))
    return api.create_page(title, 'cms_qe/home.html', language, **page_params)


# pylint: disable=dangerous-default-value
def create_text_page(title, language='en', page_params={}, plugin_params={}):
    plugin_params.setdefault('body', 'shello')
    page = create_page(title, language, page_params)
    placeholder = page.placeholders.get(slot='content')
    api.add_plugin(placeholder, 'TextPlugin', language, **plugin_params)
    return page



from webdriverwrapper.decorators import expected_error_page

from ..pages.cms import CreatePagePage


@expected_error_page('Generic error')
def test_page_not_found(driver):
    driver.go_to('/404')


# No error page should be detected.
def test_page_not_found_custom_by_cms(driver):
    CreatePagePage(driver).open().create_page('Page not found', slug='error404')
    driver.go_to('/404')



"""
Base settings for Django CMS and all plugins for Django CMS.
"""

from cms.constants import X_FRAME_OPTIONS_SAMEORIGIN
from django.utils.translation import ugettext_lazy as _

# Base Django CMS settings.

# List of base templates to create page from.
CMS_TEMPLATES = [
    ('cms_qe/home.html', 'Home page template'),
]

# Do not show toolbar for not logged users when they append ?edit into URL.
CMS_TOOLBAR_ANONYMOUS_ON = False

# Enables permission to view or edit page only by specific users.
CMS_PERMISSION = True

# Django CMS allow to configure X-FRAME header per page. We don't want to allow nothing
# else than SAMEORIGIN which is set also by Django's setting X_FRAME_OPTIONS.
CMS_DEFAULT_X_FRAME_OPTIONS = X_FRAME_OPTIONS_SAMEORIGIN

# Caching.
CMS_PAGE_CACHE = True
CMS_PLACEHOLDER_CACHE = True
CMS_PLUGIN_CACHE = True

# Enables to set permissions per any folder or file for specific users or groups.
FILER_ENABLE_PERMISSIONS = True

# Django Filer settings.

THUMBNAIL_HIGH_RESOLUTION = True

THUMBNAIL_PROCESSORS = (
    'easy_thumbnails.processors.colorspace',
    'easy_thumbnails.processors.autocrop',
    'filer.thumbnail_processors.scale_and_crop_with_subject_location',
    'easy_thumbnails.processors.filters'
)

TEXT_ADDITIONAL_TAGS = ('iframe',)

# cmsplugin_filer_folder
CMSPLUGIN_FILER_FOLDER_STYLE_CHOICES = (
    ("list", _("List")),
    ("slideshow", _("Slideshow")),
    ("gallery", _("Gallery"))
)

CMSPLUGIN_FILER_IMAGE_DEFAULT_STYLE = 'main'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-08 09:15
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms_qe_auth', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'ordering': ('username',)},
        ),
    ]



import re

from django.contrib.auth import get_user_model
from django.test import override_settings
from pytest_data import use_data


@use_data(user_data={'username': 'testuser', 'password': 'testpass'})
def test_login(client, user):
    res = client.post('/en/auth/login/', {'username': 'testuser', 'password': 'testpass'})
    assert res.status_code == 302


def test_register(mailoutbox, client):
    assert len(mailoutbox) == 0
    assert not get_user_model().objects.filter(username='testuser')

    user = _register_user(client)

    assert user.email == 'testuser@example.com'
    assert len(mailoutbox) == 1
    activation_mail = mailoutbox[0]
    assert 'activate' in activation_mail.body
    assert 'http' in activation_mail.body


@override_settings(AUTHENTICATION_BACKENDS=[
    'django.contrib.auth.backends.ModelBackend',
    'cms_qe_auth.tests.utils.TestAuthBackend',
])
def test_activation_multiple_authentication_backends(client, mailoutbox):
    _test_activation(client, mailoutbox)


def test_activation(client, mailoutbox):
    _test_activation(client, mailoutbox)


def _test_activation(client, mailoutbox):
    user = _register_user(client)
    assert not user.is_active

    # Get activation link from email
    activation_mail = mailoutbox[0]
    activate_url_pattern = '(?P<url>https?://[^\s]+/activate/[^\s]+)'
    url = re.search(activate_url_pattern, activation_mail.body).group('url')

    response = client.get(url)
    user.refresh_from_db()

    assert user.is_active
    # Test automatic login
    assert response.context['user'].is_authenticated


def _register_user(client):
    res = client.post('/en/auth/register/', {
        'username': 'testuser',
        'password1': 'testpass',
        'password2': 'testpass',
        'email': 'testuser@example.com',
    })
    assert res.status_code == 302
    return get_user_model().objects.get(username='testuser')



"""
Database settings, used PostgreSQL without auth by default.
"""

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'cms_qe',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'OPTIONS': {
            'application_name': 'cms_qe',
        }
    }
}



"""
Big goal is to have this library secure, so it has by default those main settings
ready:

 * Used HTTPS with HSTS with all subdomains (but no preload).
 * Session cookie is secure and readable by HTTP only.
 * Page can be used in iframe only on the same domain (turn on only because of Django CMS).
 * CSP header to not allow any not trusted content.
 * XSS-protection header to not allow run any script submitted by form.
 * Content-type: nosniff header to not allow run any page or script uploaded as a file.
 * Use CSRF to not allow make dangerous post request outside of the page.
 * :ref:`Save password very securely by latest best algorithm
   <django:auth_password_storage>`.
 * Use password validators to allow at least 8 characters which should not be
   user name, common password or all numeric.
 * Use ``django-axes`` to limit authorization (after 5 attempts is username and IP banned
   for one hour, can be changed by custom configuration).

Tip: when you want to just test page without final configuration, it's good to temporally
disable CSP header in your config (actually - just report but don't block):

.. code-block:: python

    CSP_REPORT_ONLY = True

If your test environment do not run with SSL, then you should turn it off:

.. code-block:: python

    META_SITE_PROTOCOL = 'http'
    SESSION_COOKIE_SECURE = False
    SECURE_HSTS_SECONDS = 0

If your website is behind proxy, you have to set Django AXES to properly limit brute-force
authorization by one of those options (more about that in `documentation
<https://django-axes.readthedocs.io/en/latest/configuration.html>`_:

.. code-block:: python

    AXES_REVERSE_PROXY_HEADER = 'header'
    AXES_NUM_PROXIES = 1

"""

from typing import List  # pylint: disable=unused-import

# Cookies.

SESSION_COOKIE_NAME = 'sessionid'
SESSION_COOKIE_SECURE = True

# Secure headers.

X_FRAME_OPTIONS = 'SAMEORIGIN'  # Django CMS needs SAMEORIGIN to allow in-page edit.

SECURE_BROWSER_XSS_FILTER = True

SECURE_CONTENT_TYPE_NOSNIFF = True

SECURE_HSTS_SECONDS = 31536000
SECURE_HSTS_PRELOAD = False
SECURE_HSTS_INCLUDE_SUBDOMAINS = True

# googleapis and gstatic is because of fonts and other libraries (js, maps) hosted at Google's CDN.
# githubusercontent.com/divio is because of Django CMS which checks in admin if there is some update.
CSP_DEFAULT_SRC = ("'self'",)
CSP_SCRIPT_SRC = ("'self'", "'unsafe-inline'", "'unsafe-eval'", '*.googleapis.com', '*.gstatic.com')
CSP_STYLE_SRC = ("'self'", "'unsafe-inline'", '*.googleapis.com')
# Google maps use fonts as data
CSP_FONT_SRC = ("'self'", "data:", 'fonts.gstatic.com')
CSP_IMG_SRC = ("'self'", "data:", '*.googleapis.com', '*.gstatic.com')
CSP_CONNECT_SRC = ("'self'", '*.django-cms.org', 'raw.githubusercontent.com/divio')
CSP_FRAME_SRC = ("'self'", 'https:')
CSP_REPORT_URI = '/csp-report'

# CSRF.

CSRF_FAILURE_VIEW = 'cms_qe.views.errors.handler403'
CSRF_HEADER_NAME = 'HTTP_X_CSRFTOKEN'
CSRF_TRUSTED_ORIGINS = []  # type: List[str]
CSRF_USE_SESSIONS = True  # Since Django 1.11

# Password validation.
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        'OPTIONS': {
            'min_length': 8,
        },
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Rewritten password hashes with Argon2
PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
]

# Authentication limits.
# https://django-axes.readthedocs.io/en/latest/configuration.html

AXES_LOGIN_FAILURE_LIMIT = 5
AXES_LOCK_OUT_AT_FAILURE = True
AXES_USE_USER_AGENT = True
AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP = True

AXES_COOLOFF_TIME = 1  # hour
AXES_LOCKOUT_TEMPLATE = 'cms_qe/auth/locked.html'

AXES_DISABLE_ACCESS_LOG = False
AXES_DISABLE_SUCCESS_ACCESS_LOG = True



from django.core.paginator import Page
from pytest_data import use_data


def test_get_header(cms_qe_table_model):
    assert cms_qe_table_model.get_header() == ['username', 'password']


@use_data(cms_qe_table_model_data={'paging_show': False})
def test_get_items_without_paging(cms_qe_table_model):
    assert cms_qe_table_model.get_items() == []


@use_data(cms_qe_table_model_data={'paging_show': True})
def test_get_items_with_paging(cms_qe_table_model):
    items = cms_qe_table_model.get_items()
    assert isinstance(items, Page)
    assert list(items) == []


@use_data(cms_qe_table_model_data={'columns': ['does_not_exist']})
def test_columns_exist(cms_qe_table_model):
    assert not cms_qe_table_model.columns_exist



from cms.models import CMSPlugin
from django.core.exceptions import ValidationError
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


SUBSCRIBE = 1
UNSUBSCRIBE = 2

SERVICE_NONE = 1
SERVICE_MAILCHIMP = 2
SERVICES = (
    (SERVICE_NONE, _('None')),
    (SERVICE_MAILCHIMP, _('Mailchimp')),
)


class NewsletterPluginModel(CMSPlugin):
    """
    Configuration model for newsletter plugin.
    Allows to set:
     * Title
     * Mail lists on mailchimp to which subscribers will be added
     * Show or hide full name
     * Require full name or not
    """

    mailing_lists = models.ManyToManyField(
        'MailingList',
        verbose_name=_('Maillist'),
    )
    title = models.CharField(
        max_length=150,
        verbose_name=_('Title'),
        help_text=_('The title of subscribes that will be shown to users'),
        blank=True
    )
    fullname_show = models.BooleanField(
        verbose_name=_('Show fullname fields'),
        default=False,
    )
    fullname_require = models.BooleanField(
        verbose_name=_('Require fullname'),
        default=False,
    )

    def __str__(self):
        return self.title

    def clean(self):
        if not self.fullname_show and self.fullname_require:
            raise ValidationError(_('Can not hide and require a full name at the same time.'))


class MailingList(models.Model):
    """
    Mailing list model which also synchronize with external services like Mailchimp.
    If ``external_id`` is not set then subscriber is not synchronized.
    """

    name = models.CharField(max_length=50)

    external_service = models.PositiveSmallIntegerField(choices=SERVICES, default=SERVICE_NONE)
    external_id = models.CharField(max_length=10, null=True, blank=True)

    def __str__(self):
        return self.name

    def clean(self):
        if self.external_service != SERVICE_NONE and not self.external_id:
            raise ValidationError(_('External ID is required when external service selected.'))


class Subscriber(models.Model):
    """
    Subscriber model which also synchronize with external services like Mailchimp.
    If ``external_id`` is not set then subscriber is not synchronized.
    """

    mailing_list = models.ForeignKey(MailingList)
    email = models.EmailField()
    first_name = models.CharField(max_length=50, blank=True)
    last_name = models.CharField(max_length=50, blank=True)

    external_id = models.CharField(max_length=50, null=True, blank=True)

    def __str__(self):
        # pylint: disable=no-member
        return _('{} in list {}').format(self.email, self.mailing_list)

    # pylint: disable=arguments-differ
    def save(self, *args, **kwargs):
        self.subscribe()
        super(Subscriber, self).save(*args, **kwargs)

    # pylint: disable=arguments-differ
    def delete(self, *args, **kwargs):
        self.unsubscribe()
        super(Subscriber, self).delete(*args, **kwargs)

    def subscribe(self):
        """
        Called before save to create task to sync action with external service.
        """
        if self.mailing_list.external_service == SERVICE_NONE:
            return

        if not self.external_id:
            SubscribeTask.objects.create(
                mailing_list=self.mailing_list,
                email=self.email,
                first_name=self.first_name,
                last_name=self.last_name,
                type=SUBSCRIBE,
            )

    def unsubscribe(self):
        """
        Called before delete to create task to sync action with external service.
        """
        if self.mailing_list.external_service == SERVICE_NONE:
            return

        if self.external_id:
            SubscribeTask.objects.create(
                mailing_list=self.mailing_list,
                email=self.email,
                first_name=self.first_name,
                last_name=self.last_name,
                external_id=self.external_id,
                type=UNSUBSCRIBE,
            )
        else:
            # Not sync, just for case cancel any sync task.
            SubscribeTask.objects.filter(
                mailing_list=self.mailing_list,
                email=self.email,
            ).delete()


class SubscribeTask(models.Model):
    """
    Item for task queue for subscribing and unsubscribing members on the external
    service like Mailchimp. The queue can is processed by ``cms-qe-newsletter-sync`` command.
    """

    TASK_TYPES = (
        (SUBSCRIBE, _('Subscribe')),
        (UNSUBSCRIBE, _('Unsubscribe')),
    )

    # The subscriber's parameters are copied to the task,
    # in order not to lose them if the subscriber is removed from the database
    mailing_list = models.ForeignKey(MailingList, null=True)
    email = models.EmailField()
    first_name = models.CharField(max_length=50, null=True)
    last_name = models.CharField(max_length=50, null=True)
    external_id = models.CharField(max_length=10, null=True)

    # Task params:
    type = models.PositiveSmallIntegerField(choices=TASK_TYPES)
    created = models.DateTimeField(auto_now=True)
    attempts = models.PositiveSmallIntegerField(default=0)
    last_error = models.TextField()

    def __str__(self):
        # pylint: disable=no-member
        return _('Task for email {}').format(self.email)

    def failure(self, error_message):
        self.attempts += 1
        self.last_error = error_message
        self.save()

    def should_process(self) -> bool:
        """
        Checks whether task should be tried to process. With less attempts
        it tries more often, with more attempts it waits more time to not
        overwhelm resources.
        """
        time_to_process = self.created + timezone.timedelta(seconds=self.attempts ** 4)
        return time_to_process <= timezone.now()



from django.core.exceptions import ObjectDoesNotExist
from requests.exceptions import HTTPError

from . import mailchimp
from ..models import SERVICE_MAILCHIMP, SUBSCRIBE, UNSUBSCRIBE, Subscriber, SubscribeTask


SERVICE_TO_MODULE = {
    SERVICE_MAILCHIMP: mailchimp,
}


def sync_mailing_lists():
    """
    Download mailing lists from external services like Mailchimp.
    """
    mailchimp.sync_mailing_lists()


def sync_tasks():
    """
    Processing tasks yielding same tuple as :py:func:`~.sync_task` function.
    """
    tasks = SubscribeTask.objects.all()
    for task in tasks:
        try:
            result, message = sync_task(task)

        # pylint: disable=broad-except
        except Exception as exc:
            result, message = False, str(exc)

        finally:
            if result is False:
                task.failure(message)
            yield (result, '{}: {}'.format(task, message))


# pylint: disable=too-many-return-statements
def sync_task(task):
    """
    Processing task returning tuple of status (``True`` for success, ``False`` for
    some failure and ``None`` for warnings) and message.
    """
    if not task.should_process():
        return None, 'Skipped'

    service = SERVICE_TO_MODULE.get(task.mailing_list.external_service)
    if not service:
        return False, 'Unsupported service {}'.format(task.mailing_list.external_service)

    try:
        if task.type == SUBSCRIBE:
            sync_subscribe(service, task)
        elif task.type == UNSUBSCRIBE:
            sync_unsubscribe(service, task)

    except HTTPError as exc:
        text = exc.response.text
        if 'fake or invalid' in text:
            Subscriber.objects.filter(email=task.email).all().delete()
            task.delete()
            return None, 'Email is invalid or fake and is deleted\n{}'.format(text)
        return False, '{}\n{}'.format(text, exc)

    except ObjectDoesNotExist:
        task.delete()
        return None, 'Subscriber does not exist anymore, deleting task'

    # pylint: disable=broad-except
    except Exception as exc:
        return False, str(exc)

    else:
        task.delete()
        return True, 'OK'


def sync_subscribe(service, task):
    subscriber = Subscriber.objects.get(mailing_list=task.mailing_list, email=task.email)

    external_id = service.sync_subscribe(
        task.mailing_list.external_id,
        task.email,
        task.first_name,
        task.last_name,
    )

    subscriber.external_id = external_id
    subscriber.save()


def sync_unsubscribe(service, task):
    service.sync_unsubscribe(
        task.mailing_list.external_id,
        task.email,
    )



import re
from pytest_data import use_data

from cms_qe_test import render_plugin

from ..cms_plugins import HostingVideoPlayerPlugin, SourceFileVideoPlayerPlugin


@use_data(cms_qe_video_source_file_video_player_model_data={'source_file': None})
def test_render_source_file_video_plugin_without_source_file(cms_qe_video_source_file_video_player_model):
    html = render_plugin(SourceFileVideoPlayerPlugin, cms_qe_video_source_file_video_player_model)
    assert re.search(r'<p>Video file is missing</p>', html)


def test_render_source_file_video_plugin(cms_qe_video_source_file_video_player_model):
    html = render_plugin(SourceFileVideoPlayerPlugin, cms_qe_video_source_file_video_player_model)
    assert not re.search(r'<iframe(\s|.)*</iframe>', html)


def test_render_hosting_video_plugin(cms_qe_video_hosting_video_player_model):
    html = render_plugin(HostingVideoPlayerPlugin, cms_qe_video_hosting_video_player_model)
    assert not re.search(r'<video(\s|.)*</video>', html)
    assert re.search(r'<iframe(\s|.)*</iframe>', html)



from typing import Iterable, Dict, Any

from django.utils.translation import ugettext as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms_qe_newsletter.forms import SubscriberForm

from .models import NewsletterPluginModel


@plugin_pool.register_plugin
class NewsletterPlugin(CMSPluginBase):
    """
    Newsletter Django-CMS plugin. Allow add the form with subscriptions on a page.
    """
    model = NewsletterPluginModel
    name = _('Newsletter subscription form')
    render_template = 'cms_qe/newsletter/newsletter.html'
    fieldsets = [
        (None, {
            'fields': (
                'title',
                'mailing_lists',
                'fullname_show',
                'fullname_require',
            )
        })
    ]
    cache = False

    def render(self, context: Dict[str, Any], instance: NewsletterPluginModel, placeholder) -> Iterable:
        request = context['request']
        if request.method == 'POST':
            form = SubscriberForm(fullname_require=instance.fullname_require, data=request.POST)
            if form.is_valid():
                form.save(instance.mailing_lists.all())
                form.saved = True
        else:
            form = SubscriberForm(fullname_require=instance.fullname_require)
        context.update({
            'instance': instance,
            'form': form,
        })
        return context



from cms_qe_test.cms import create_page


def test_page_found(client):
    create_page('Test page', page_params={'slug': 'test'})
    html = client.get('/en/test/').content
    assert b'<h1>Generic error</h1>' not in html
    assert b'<title>Test page</title>' in html


def test_page_not_found(client):
    html = client.get('/en/non-existing-page/').content
    assert b'<h1>Generic error</h1>' in html
    assert b'error404' in html


def test_page_not_found_custom_by_cms(client):
    create_page('custom page not found', page_params={'slug': 'error404'})
    html = client.get('/en/non-existing-page/').content
    assert b'<h1>Generic error</h1>' not in html
    assert b'<title>custom page not found</title>' in html



"""
Settings which is able to set from admin by user.
"""
from collections import OrderedDict

from django.utils.safestring import mark_safe


CONSTANCE_BACKEND = 'constance.backends.database.DatabaseBackend'

CONSTANCE_DATABASE_CACHE_BACKEND = 'default'

CONSTANCE_ADDITIONAL_FIELDS = {
    'short_str': ['django.forms.fields.CharField', {'required': False}],
}


GOOGLE_ANALYTICS_CONSTANCE_CONFIG = (
    ('GOOGLE_ANALYTICS_PROPERTY_ID', (
        '',
        'Every website you track with Google Analytics gets its own property ID.\n'
        'You can find the web property ID on the overview page of your account.\n',
        'short_str'
    )),
    ('GOOGLE_ANALYTICS_DISPLAY_ADVERTISING', (
        False,
        'Display Advertising allows you to view Demographics and Interests reports,\n'
        'add Remarketing Lists and support DoubleClick Campain Manager integration.',
        bool
    )),
    ('GOOGLE_ANALYTICS_SITE_SPEED', (
        False,
        mark_safe('Allow you view page load times in the '
                  '<a href="https://support.google.com/analytics/answer/1205784">Site Speed</a> report.'),
        bool
    )),
    ('GOOGLE_ANALYTICS_ANONYMIZE_IP', (
        False,
        mark_safe('<a href="https://support.google.com/analytics/answer/2763052?hl=en">IP Anonymization</a>'),
        bool
    )),
    ('GOOGLE_ANALYTICS_SAMPLE_RATE', (
        '',
        mark_safe(
            '<a href="https://developers.google.com/analytics/devguides/collection/gajs/methods/'
            'gaJSApiBasicConfiguration#_setsamplerate">Sample Rate </a>'
        ),
        'short_str'
    )),
    ('GOOGLE_ANALYTICS_SITE_SPEED_SAMPLE_RATE', (
        '',
        mark_safe(
            '<a href="https://developers.google.com/analytics/devguides/collection/gajs/methods/'
            'gaJSApiBasicConfiguration#_setsitespeedsamplerate">Site Speed Sample Rate</a>\n'
            'The value is a percentage and can be between 0 and 100'
        ),
        'short_str')),
    ('GOOGLE_ANALYTICS_SESSION_COOKIE_TIMEOUT', (
        '',
        mark_safe(
            '<a href="https://developers.google.com/analytics/devguides/collection/gajs/methods/'
            'gaJSApiBasicConfiguration#_setsessioncookietimeout">Session cookie timeout</a> in milliseconds.'
        ),
        'short_str'
    )),
    ('GOOGLE_ANALYTICS_VISITOR_COOKIE_TIMEOUT', (
        '',
        mark_safe(
            '<a href="https://developers.google.com/analytics/devguides/collection/gajs/methods/'
            'gaJSApiBasicConfiguration#_setvisitorcookietimeout">Google Analytics visitor cookie expiration </a>'
            'in milliseconds.'),
        'short_str'
    )),
)

PIWIK_CONSTANCE_CONFIG = (
    ('PIWIK_DOMAIN_PATH', (
        '',
        'URL of your Piwik server. Typically, you’ll have Piwik installed on a subdomain or subdirectory\n '
        '(e.g. piwik.your_site.com or www.your_site.com/piwik).',
        'short_str'
    )),
    ('PIWIK_SITE_ID', (
        '',
        'Your Piwik server can track several websites.\n Each website has its site ID (this is the idSite parameter '
        'in the query string of your browser’s address bar when you visit the Piwik Dashboard). ',
        'short_str'
    )),
)

DJANGOCMS_GOOGLEMAP_CONSTANCE_CONFIG = (
    ('DJANGOCMS_GOOGLEMAP_API_KEY', (
        '',
        'API key of your project to use Google maps. You can generate key here:\n'
        'https://developers.google.com/maps/documentation/javascript/get-api-key'
    )),
)

MAILCHIMP_CONSTANCE_CONFIG = (
    ('MAILCHIMP_API_KEY', (
        '',
        'API key of your app to synchronize mail lists with MailChimp. More about MailChimp API keys:'
        'http://kb.mailchimp.com/integrations/api-integrations/about-api-keys',
    )),
    ('MAILCHIMP_USERNAME', (
        '',
        'Your username on MailChimp. You can find or change the username in profile settings:'
        'https://admin.mailchimp.com/account/profile/',
    )),
)

CONSTANCE_CONFIG = OrderedDict(
    GOOGLE_ANALYTICS_CONSTANCE_CONFIG +   # type: ignore
    PIWIK_CONSTANCE_CONFIG +   # type: ignore
    DJANGOCMS_GOOGLEMAP_CONSTANCE_CONFIG +   # type: ignore
    MAILCHIMP_CONSTANCE_CONFIG  # type: ignore
)

CONSTANCE_CONFIG_FIELDSETS = {
    'General options': ('DJANGOCMS_GOOGLEMAP_API_KEY',),
    'Google analytics options': dict(GOOGLE_ANALYTICS_CONSTANCE_CONFIG).keys(),
    'Piwik options': dict(PIWIK_CONSTANCE_CONFIG).keys(),
    'Mailchimp options': dict(MAILCHIMP_CONSTANCE_CONFIG).keys(),
}


def _lazy_constance(constance_name):
    """
    Config from library constance cannot be imported in settings
    because it already need initialized settings. That's why it
    has to be done lazily.
    """

    def get_constance_value():
        """
        Simple function just to import config and get value. This
        function will be called later when settings is alread initialized.
        """
        from constance import config  # pylint: disable=import-error
        return getattr(config, constance_name)

    # pylint: disable=too-few-public-methods
    class Template:
        """
        Simple objcet wrapping real value. It's just proxy to real
        value. It works only with values which are not validated
        against realy base data types.
        """

        def __getattr__(self, item):
            return getattr(get_constance_value(), item)

        def __str__(self):
            return str(get_constance_value())

    return Template()


DJANGOCMS_GOOGLEMAP_API_KEY = _lazy_constance('DJANGOCMS_GOOGLEMAP_API_KEY')



from importlib import import_module
from typing import Iterable, Tuple, Union

from django.apps import apps
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.template import TemplateDoesNotExist
from django.template.loader import get_template

from mailqueue.models import MailerMessage


# pylint:disable=invalid-name
def get_email(template: str, subject: str, to: Union[str, Iterable[str]], from_email: str = None, **kwargs):
    """
    Returns a ``MailerMessage`` instance from  ``mailqueue``. Use ``save()`` method instead of ``send()`` to send
    message or put it to a mailqueue.

    Template should be without extension and you should create both ``.txt`` and ``.html`` version.
    Second one is not mandatory but is good to provide it as well.

    """

    email = MailerMessage()
    email.subject = subject
    email.from_address = from_email or settings.DEFAULT_FROM_EMAIL

    if isinstance(to, str):
        to = [to]
    email.to_address = ', '.join(to)

    template_txt = get_template(template + '.txt')
    content = template_txt.render(kwargs)
    email.content = content

    try:
        template_html = get_template(template + '.html')
    except TemplateDoesNotExist:
        return email
    email.html_content = template_html.render(kwargs)

    return email


def get_base_url(request) -> str:
    """
    Helper to get absolute URL of application. It requires to set
    correctly domain of site framework.
    """
    protocol = settings.META_SITE_PROTOCOL
    domain = get_current_site(request)
    return '{}://{}'.format(protocol, domain)


def get_functions(module_name: str, function_name: str) -> Iterable[Tuple[str, object]]:
    """
    Get function by ``function_name`` of ``module_name`` for every installed
    Django app. Returns tuple of ``app_name`` and ``function``. Example usage:

    .. code-block:: python

        for app, func in get_functions('monitoring', 'get_status'):
            # ...
    """
    for app_name, module in get_modules(module_name):
        function = getattr(module, function_name, None)
        if function:
            yield (app_name, function)


def get_modules(module_name: str) -> Iterable[Tuple[str, object]]:
    """
    Get module by ``module_name`` for every installed Django app.
    Returns tuple of ``app_name`` and ``module``. Example usage:

    .. code-block:: python

        for app, module in get_modules('models'):
            # ...
    """
    for app in apps.get_app_configs():
        module = get_module(app.name, module_name)
        if module:
            yield (app.name, module)


def get_module(app_name: str, module_name: str) -> object:
    """
    Helper to load module by ``module_name`` of Django app ``app_name``.
    Returns ``None`` if module does not exist.
    """
    import_module(app_name)  # The app has to exists.
    module_name = '{}.{}'.format(app_name, module_name)
    try:
        return import_module(module_name)
    except ImportError:
        return None



from typing import Any, Dict, List, Tuple, Union

from cms.models.pluginmodel import CMSPlugin
from django.core.paginator import Paginator, Page, EmptyPage, PageNotAnInteger
from django.db import models
from django.db.models.base import ModelBase
from django.utils.translation import ugettext as _
from jsonfield import JSONField

from .exceptions import TableDoesNotExists
from .utils import get_model_by_table, get_filter_params


class TablePluginModel(CMSPlugin):
    """
    Configuration model for plugin Table for Django CMS QE. Possibility to specify
    table, columns, order, filtering etc. etc.
    """

    table = models.CharField(max_length=100, verbose_name=_('Table to show'))
    columns = JSONField(verbose_name=_('Columns to show'))
    filter = JSONField(verbose_name=_('Filter'))
    paging_show = models.BooleanField(default=True, verbose_name=_('Show paging'))
    paging_per_page = models.IntegerField(default=20, verbose_name=_('How many items per page when paging'))

    def __str__(self):
        if self.table_exists:
            return '{m.app_label} / {m.object_name}'.format(m=self.model._meta)
        return _('Non existing table {}.').format(self.table)

    @property
    def table_exists(self) -> bool:
        """
        Returns if table exists. When programmer move model or rename table
        without change in this configuration, it will not be available anymore.
        """
        try:
            get_model_by_table(self.table)
            return True
        except TableDoesNotExists:
            return False

    @property
    def columns_exist(self) -> bool:
        """
        Returns if all columns exists. When programmer rename column
        without change in this configuration, it will not be available anymore.
        """
        if not self.table_exists:
            return False
        model = self.model
        # pylint:disable=not-an-iterable
        if not all(hasattr(model, column) for column in self.columns):
            return False
        return True

    @property
    def model(self) -> ModelBase:
        """
        Returns model for configured table.
        """
        return get_model_by_table(self.table)

    def get_header(self) -> List[str]:
        """
        Returns header for table with ``verbose_name`` of fields if exists.
        In other cases at least uses name of that field.
        """
        def f(column):
            field = self.model._meta.get_field(column)
            return getattr(field, 'verbose_name', field.name)
        return [f(column) for column in self.columns]  # pylint: disable=not-an-iterable

    def get_filter_params(self) -> Tuple[List[Any], Dict[str, Any]]:
        """
        Returns tuple with args and kwargs for queryset filter.
        """
        return get_filter_params(self.model, self.filter)

    def get_items(self, page: Union[int, str] = None) -> Page:
        """
        Returns items for table without header. It's simply list (items)
        of lists (columns), not whole objects. Header is not included,
        for that use :any:`TablePluginModel.get_header`.
        """
        items_list = self.model.objects
        if self.filter:
            filter_args, filter_kwds = self.get_filter_params()
            items_list = items_list.filter(*filter_args, **filter_kwds)

        items_list = items_list.all()
        if not self.paging_show:
            return self._get_items(items_list)

        paginator = Paginator(items_list, self.paging_per_page)

        try:
            items = paginator.page(page)
        except PageNotAnInteger:
            items = paginator.page(1)
        except EmptyPage:
            items = paginator.page(paginator.num_pages)

        # Evaluate query and return only needed columns.
        items.object_list = self._get_items(items)
        return items

    def _get_items(self, items: List[ModelBase]) -> List[List[str]]:
        return [
            [getattr(item, column, '') for column in self.columns]  # pylint: disable=not-an-iterable
            for item in items
        ]



import pytest
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory
from pytest_data import get_data

from cms_qe.utils import get_email


@pytest.fixture
def post_request_empty(request):
    rf = RequestFactory()
    rq = rf.post('', get_data(request, 'post_request_empty_data', {}))
    setattr(rq, 'session', True)
    messages = FallbackStorage(rq)
    setattr(rq, '_messages', messages)
    return rq


@pytest.fixture
def email(request):
    data = get_data(
        request, 'email_data', {
            'template': 'cms_qe/tests/test_email',
            'subject': 'Test',
            'to': 'test@example.com',
            'content': 'Test content'
        }
    )
    return get_email(**data)



from constance.test import override_config
from django.conf import settings
import pytest
from pytest_data import use_data

from ..external_services.mailchimp import sync_mailing_lists, sync_subscribe, sync_unsubscribe
from ..models import MailingList


@pytest.mark.slow
@override_config(
    MAILCHIMP_USERNAME=settings.TEST_MAILCHIMP_USERNAME,
    MAILCHIMP_API_KEY=settings.TEST_MAILCHIMP_API_KEY,
)
def test_sync_mailing_lists_integrate():
    # Check connection with MailChimp.
    _test_sync_mailing_lists()


def test_sync_mailing_lists(mock_mailchimp):
    _test_sync_mailing_lists()


def _test_sync_mailing_lists():
    assert MailingList.objects.count() == 0
    sync_mailing_lists()
    assert MailingList.objects.count() == 2


@use_data(cms_qe_mailchimp_subscribe_data={'id': 'subid'})
def test_sync_subscribe(mock_mailchimp):
    assert 'subid' == sync_subscribe('listid', 'test@example.com', 'first', 'last')


def test_sync_unsubscribe(mock_mailchimp):
    # No exception.
    sync_unsubscribe('listid', 'test@example.com')



from django.http import HttpRequest
from django.utils.translation import ugettext_lazy as _
from menus.base import Modifier, NavigationNode
from menus.menu_pool import menu_pool


URL_NAME_TO_TITLE = {
    'login': _('Login'),
    'logout': _('Logout'),
    'password_change': _('Change password'),
    'password_change_done': _('Change password'),
    'password_reset': _('Reset pasword'),
    'password_reset_done': _('Reset pasword'),
    'password_reset_confirm': _('Reset pasword'),
    'password_reset_complete': _('Reset pasword'),
    'register': _('Register'),
}


@menu_pool.register_modifier
class AuthModifier(Modifier):
    """
    This modifier adds to breadcrumb all authentication URLs.
    """
    # pylint: disable=too-many-arguments
    def modify(
            self,
            request: HttpRequest,
            nodes: NavigationNode,
            namespace: str,
            root_id: int,
            post_cut: bool,
            breadcrumb: bool,
    ) -> NavigationNode:
        if not (
                breadcrumb
                and request.resolver_match
                and request.resolver_match.url_name in URL_NAME_TO_TITLE
        ):
            return nodes

        root_nodes = [node for node in nodes if node.attr.get('is_home')]
        if root_nodes:
            root = root_nodes[0]
        else:
            root = NavigationNode('Home', '/', 'rootnodeid')
            root.selected = True

        title = URL_NAME_TO_TITLE[request.resolver_match.url_name]
        node = NavigationNode(title, request.path, 'extranodeid')
        node.selected = True
        node.parent = root
        root.children = [node]

        return [root, node]
